Package: editbl
Type: Package
Version: 1.3.2
Date: 2026-07-09
Title: 'DT' Extension for CRUD (Create, Read, Update, Delete)
        Applications in 'shiny'
Authors@R: c(person("Jasper", "Schelfhout", "", "jasper.schelfhout@openanalytics.eu",
           role = c("aut", "cre")),
           person("Maxim", "Nazarov", "", "maxim.nazarov@openanalytics.eu",
           role = c("rev")),
           person("Daan", "Seynaeve", "", "daan.seynaeve@openanalytics.eu",
           role = c("rev")),
           person("Lennart", "Tuijnder", "", "lennart.tuijnder@openanalytics.eu",
           role = c("rev")),
		   person("Saar", "Junius", "", "saar.junius@openanalytics.eu",
           role = c("aut")))
Maintainer: Jasper Schelfhout <jasper.schelfhout@openanalytics.eu>
Description: The core of this package is a function eDT() which enhances DT::datatable() such that it can be used to interactively modify data in 'shiny'. By the use of generic 'dplyr' methods it supports many types of data storage, with relational databases ('dbplyr') being the main use case.
License: GPL-3
Copyright: Open Analytics NV, 2023
Imports: shiny, shinyjs, DT, tibble, dplyr, rlang, uuid, fontawesome
        (>= 0.4.0)
Suggests: testthat, dtplyr, data.table, vctrs, RSQLite, dbplyr, glue,
        DBI, bit64, knitr, dm
URL: https://github.com/openanalytics/editbl
BugReports: https://github.com/openanalytics/editbl/issues
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-07-09 12:09:46 UTC; jschelfhout
Author: Jasper Schelfhout [aut, cre],
  Maxim Nazarov [rev],
  Daan Seynaeve [rev],
  Lennart Tuijnder [rev],
  Saar Junius [aut]
Repository: CRAN
Date/Publication: 2026-07-09 16:30:02 UTC
