# Generated by roxygen2: do not edit by hand

S3method(e_rows_delete,default)
S3method(e_rows_insert,default)
S3method(e_rows_insert,dtplyr_step)
S3method(e_rows_insert,tbl_dbi)
S3method(e_rows_update,data.frame)
S3method(e_rows_update,default)
S3method(e_rows_update,dtplyr_step)
S3method(e_rows_update,tbl_dbi)
S3method(inputServer,default)
S3method(inputUI,default)
S3method(rows_delete,dtplyr_step)
export(connectDB)
export(customButton)
export(eDT)
export(eDTOutput)
export(e_rows_delete)
export(e_rows_insert)
export(e_rows_update)
export(foreignTbl)
export(inputServer)
export(inputUI)
export(runDemoApp)
export(selectInputDT_Server)
export(selectInputDT_UI)
import(fontawesome)
importFrom(DT,DTOutput)
importFrom(DT,coerceValue)
importFrom(DT,dataTableProxy)
importFrom(DT,datatable)
importFrom(DT,formatStyle)
importFrom(DT,hideCols)
importFrom(DT,renderDT)
importFrom(DT,renderDataTable)
importFrom(DT,styleEqual)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,anti_join)
importFrom(dplyr,as_tibble)
importFrom(dplyr,collect)
importFrom(dplyr,count)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,if_any)
importFrom(dplyr,inner_join)
importFrom(dplyr,is.tbl)
importFrom(dplyr,left_join)
importFrom(dplyr,pull)
importFrom(dplyr,relocate)
importFrom(dplyr,rows_delete)
importFrom(dplyr,rows_insert)
importFrom(dplyr,rows_update)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,tbl)
importFrom(dplyr,tbl_vars)
importFrom(dplyr,tibble)
importFrom(dplyr,type_sum)
importFrom(dplyr,union)
importFrom(rlang,":=")
importFrom(shiny,HTML)
importFrom(shiny,NS)
importFrom(shiny,actionButton)
importFrom(shiny,dateInput)
importFrom(shiny,div)
importFrom(shiny,fluidPage)
importFrom(shiny,freezeReactiveValue)
importFrom(shiny,icon)
importFrom(shiny,is.reactive)
importFrom(shiny,isTruthy)
importFrom(shiny,isolate)
importFrom(shiny,modalButton)
importFrom(shiny,modalDialog)
importFrom(shiny,moduleServer)
importFrom(shiny,numericInput)
importFrom(shiny,observe)
importFrom(shiny,observeEvent)
importFrom(shiny,reactive)
importFrom(shiny,reactiveValues)
importFrom(shiny,renderPrint)
importFrom(shiny,renderUI)
importFrom(shiny,req)
importFrom(shiny,selectInput)
importFrom(shiny,shinyApp)
importFrom(shiny,showModal)
importFrom(shiny,showNotification)
importFrom(shiny,stopApp)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(shiny,textInput)
importFrom(shiny,uiOutput)
importFrom(shiny,verbatimTextOutput)
importFrom(shinyjs,disable)
importFrom(shinyjs,disabled)
importFrom(shinyjs,enable)
importFrom(shinyjs,hidden)
importFrom(shinyjs,useShinyjs)
importFrom(tibble,as_tibble)
importFrom(utils,head)
importFrom(utils,packageName)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(uuid,UUIDgenerate)
