% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eDT.R
\name{canXXXRowTemplate}
\alias{canXXXRowTemplate}
\title{Re-usable documentation}
\usage{
canXXXRowTemplate(canEditRow, canCloneRow, canDeleteRow)
}
\arguments{
\item{canEditRow}{can be either of the following:
\itemize{
\item \code{logical}, e.g. TRUE or FALSE. In case of \code{FALSE}, buttons will not be visible except for new rows.
\item \code{function}. Needs as input an argument \code{row} which accepts a single row \code{tibble} and as output TRUE/FALSE.
}}

\item{canCloneRow}{can be either of the following:
\itemize{
\item \code{logical}, e.g. TRUE or FALSE. In case of \code{FALSE}, buttons will not be visible except for new rows.
\item \code{function}. Needs as input an argument \code{row} which accepts a single row \code{tibble} and as output TRUE/FALSE.
}}

\item{canDeleteRow}{can be either of the following:
\itemize{
\item \code{logical}, e.g. TRUE or FALSE. In case of \code{FALSE}, buttons will not be visible except for new rows.
\item \code{function}. Needs as input an argument \code{row} which accepts a single row \code{tibble} and as output TRUE/FALSE.
}}
}
\description{
Re-usable documentation
}
\keyword{internal}
