% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{coalesce}
\alias{coalesce}
\title{Return first non \code{NULL} argument}
\usage{
coalesce(...)
}
\arguments{
\item{...}{set of arguments}
}
\description{
Return first non \code{NULL} argument
}
\author{
Jasper Schelfhout
}
\keyword{internal}
