% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{castToTbl}
\alias{castToTbl}
\title{Cast data to tbl}
\usage{
castToTbl(data)
}
\arguments{
\item{data}{object}
}
\value{
tbl
}
\description{
Cast data to tbl
}
\author{
Jasper Schelfhout
}
\keyword{internal}
